import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.Arrays;
/**
 * A Face on a Rubix Cube
 * 
 * NOTE: column is actually a row, row is actually a column. Weird,
 * but the names make sense for arrays and not the cube.
 * 
 * @author kiarocks
 * @version 1.5
 */
public class Face extends Actor
{
    public SquareOnFace[][] sq = new SquareOnFace[3][3];

    int imgSize = RubixWorld.CELL_SIZE*3*15;
    int imgNoCell = imgSize/RubixWorld.CELL_SIZE/3*2;
    int direction = -1;
    int oldX, oldY;
    public Face(Color faceColor, int dir) {
        for(int x = 0; x < sq.length; x++) {
            for(int y = 0; y < sq[0].length; y++) {
                sq[x][y] = new SquareOnFace(faceColor);
                sq[x][y].parent = this;
            }
        }
        direction = dir;
    }

    public int getOldX() {
        return oldX;
    }

    public int getOldY() {
        return oldY;
    }

    public void addedToWorld(World w){
        for(int x = 0; x < sq.length; x++) {
            for(int y = 0; y < sq[0].length; y++) {
                w.addObject(sq[x][y], (x*15)+getX(), (y*15)+getY());
            }
        }
        oldX = getX();
        oldY = getY();
        setLocation(getX()+imgNoCell/2,getY()+imgNoCell/2);
        GreenfootImage img = Image.createByColor(Color.black);
        img.scale(imgSize,imgSize);
        setImage(img);
    }

    public void act() {
        if(Greenfoot.mouseClicked(this)) {
            mouseClicked();
        }
    }

    public void mouseClicked() {
        MouseInfo info = Greenfoot.getMouseInfo();
        int button = info.getButton();
        if(button == 1) {
            clockwise();
        } else if (button == 3) {
            counter();
        }
    }

    /*
     * Case top:
     * Front -> left
     * Left -> back
     * Back -> right
     * Right -> front
     * 
     * Case bottom:
     * Front -> right
     * Right -> back
     * Back -> left
     * Left -> front
     */
    public void clockwise() {
        checkDirection();
        rotate();
        Face[] quads = new Face[4];
        int row = -1, column = -1;
        if(direction == RubixWorld.TOP) {
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.FRONT);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.BACK);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.RIGHT);
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.LEFT);
            row = -2;
            column = 0;
        } else if(direction == RubixWorld.BOTTOM) {
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.FRONT);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.BACK);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.RIGHT);
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.LEFT);
            row = -2;
            column = 2;
        } else if(direction == RubixWorld.FRONT) {
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.TOP);
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.RIGHT);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.BOTTOM);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.LEFT);
            row = 2;
            column = -2;
        } else if(direction == RubixWorld.BACK) {
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.LEFT);
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.BOTTOM);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.RIGHT);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.TOP);
            row = 0;
            column = -2;
        } else if(direction == RubixWorld.RIGHT) {
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.TOP);
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.BACK);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.BOTTOM);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.FRONT);
            row = 0;
            column = -2;
        } else if(direction == RubixWorld.LEFT) {
            quads[0] = RubixWorld.getFaceByDirection(RubixWorld.TOP);
            quads[1] = RubixWorld.getFaceByDirection(RubixWorld.FRONT);
            quads[2] = RubixWorld.getFaceByDirection(RubixWorld.BOTTOM);
            quads[3] = RubixWorld.getFaceByDirection(RubixWorld.BACK);
            row = 2;
            column = -2;
        } else {
            throw new IllegalStateException("Direction out of bounds");
        }
        if(row == -1 || column == -1) {
            throw new IllegalStateException("No row/column determined");
        }
        for(Face f : quads) {
            System.err.println("Face " + Strings.getDirectionAsString(direction) + " is rotating Face "
                + Strings.getDirectionAsString(f.direction));
        }
        SquareOnFace[][][] temp = new SquareOnFace[4][3][3];
        for(int x = 0; x < temp.length; x++) {
            for(int y = 0; y < temp[0].length; y++) {
                for(int z = 0; z < temp[0][0].length; z++) {
                    int mod_x = x + 1;
                    if(mod_x == 4) 
                        mod_x = 0;
                    temp[mod_x][y][z] = quads[x].sq[y][z];
                }
            }
        }
        if(column == -2) {
            if(column > -1) {
                System.out.println("Ignoring column");
            }
            for(int i = 0; i < 3; i++) {
                /* Retrive values for temp array */
                temp[0][row][i] = quads[0].sq[row][i];
                temp[1][row][i] = quads[1].sq[row][i];
                temp[2][row][i] = quads[2].sq[row][i];
                temp[3][row][i] = quads[3].sq[row][i];
            }
        } else {
            if(row > -1) {
                System.out.println("Ignoring row");
            }
            for(int i = 0; i < 3; i++) {
                /* Retrive values for temp array */
                temp[0][i][column] = quads[0].sq[i][column];
                temp[1][i][column] = quads[1].sq[i][column];
                temp[2][i][column] = quads[2].sq[i][column];
                temp[3][i][column] = quads[3].sq[i][column];
            }
        }
        /* Set new values */
        quads[0].sq = temp[1];
        quads[1].sq = temp[2];
        quads[2].sq = temp[3];
        quads[3].sq = temp[0];
        RubixWorld.update();
    }

    public void counter() {
        // Lazy counter-clockwise implementaion, is a clockwise 3 times.
        clockwise();
        clockwise();
        clockwise();
    }

    private void checkDirection() {
        if(direction == -1) {
            throw new IllegalStateException("This face has no direction associated with it");
        }
    }

    public void addTitle(Actor title) {
        if(getWorld() == null) {
            throw new IllegalStateException("World is null!");
        }
        getWorld().addObject(title, getX(), getY()-imgNoCell+5);
    }

    public void rotate() {
        SquareOnFace[][] temp = new SquareOnFace[3][3];

        //Center is same
        temp[1][1] = sq[1][1];
        //Hard coded because algorithm is hard.

        /* Corners */
        temp[0][0] = sq[2][0]; //Corner 1
        temp[0][2] = sq[0][0]; //Corner 2
        temp[2][2] = sq[0][2]; //Corner 3
        temp[2][0] = sq[2][2]; //Corner 4

        /* Edges */
        temp[0][1] = sq[1][0];
        temp[1][0] = sq[2][1];
        temp[1][2] = sq[0][1];
        temp[2][1] = sq[1][2];

        // Set new array here
        sq = temp;
    }
}
